searchNodes=[{"doc":"Solvent is an event bus built to be fast and easy-to-use, and takes a lot of inspiration from the CloudEvents spec for the best interoperability with other event systems. In the CloudEvents specification, every event is required to have an ID, a source, and a type. The source field identifies the system that sent the event, and the ID must identify the event and be unique in the scope of the source. Lastly, the type field, also called a &quot;topic,&quot; identifies the kind of event that took place. In Solvent, only the type field is required, but it is strongly recommended to provide a source field as well. Solvent generates a version 7 UUID for event ID fields and this rarely needs to be overridden. See the Solvent.Event docs for details on what defaults are provided. Subscribe to the event stream with subscribe/2 , and publish events using publish/2 . You can also create a Solvent.Subscription or a Solvent.Event yourself and pass those to subscribe/1 and publish/1 , respectively. The functions here share a signature with their corresponding struct functions, but have the additional benefit of interacting with the event bus. Telemetry [:solvent, :event, :published] Description: Emitted when an event is published Measurements: %{} Metadata: %{event_source: String.t(), event_id: String.t(), event_type: String.t(), subscriber_count: non_neg_integer()} [:solvent, :subscriber, :processing, :start] Description: Emitted when a subscriber begins processing an event Measurements: %{} Metadata: %{subscriber_id: String.t(), event_source: String.t(), event_id: String.t(), event_type: String.t()} [:solvent, :subscriber, :processing, :stop] Description: Emitted when a subscriber finishes processing an event Measurements: %{duration: non_neg_integer()} Metadata: %{subscriber_id: String.t(), event_source: String.t(), event_id: String.t(), event_type: String.t()} [:solvent, :subscriber, :subscribing, :start] Description: Emitted when a subscriber begins subscribing to the event stream Measurements: %{} Metadata: %{subscriber_id: String.t(), filter: String.t()} [:solvent, :subscriber, :subscribing, :stop] Description: Emitted when a subscriber is finished subscribing to the event stream Measurements: %{duration: non_neg_integer()} Metadata: %{subscriber_id: String.t(), filter: String.t()}","ref":"Solvent.html","title":"Solvent","type":"module"},{"doc":"","ref":"Solvent.html#build_filters/1","title":"Solvent.build_filters/1","type":"function"},{"doc":"Publish an event to the event bus, triggering all subscriber functions. Only a type (AKA &quot;topic&quot;) is required. All other fields can be supplied using the options. See Solvent.Event for details on what that struct contains. All values given as options are inserted into the event struct. ID values are version 7 UUIDs by default, and you don't need to provide them. I would recommend using the CloudEvents format for your event's type field, which starts with a reversed DNS name, and is dot-separated. This will help avoid collisions with events from other applications. Solvent . publish ( &quot;io.github.cantido.myevent.published&quot; ) { :ok , &quot;0b06bdb7-06a7-4df9-a825-1fd225ceea43&quot; } It is also recommended to provide a source option to identify your application, especially if your subscription did not specify a source . Otherwise, all events will be published with a default source, and subscriptions will be triggered for every event traveling through Solvent (including events from other applications). A good source value is either a fully-qualified domain name, or a UUID in URN format. iex&gt; Solvent . publish ( ...&gt; &quot;io.github.cantido.documentation.read&quot; , ...&gt; id : &quot;read-docs-id&quot; , ...&gt; source : &quot;myapp&quot; ...&gt; ) { :ok , { &quot;myapp&quot; , &quot;read-docs-id&quot; } } You can also build an event yourself with Solvent.Event.new/1 and publish it with this function.","ref":"Solvent.html#publish/2","title":"Solvent.publish/2","type":"function"},{"doc":"Subscribe to the event stream with a pre-made Solvent.Subscription struct.","ref":"Solvent.html#subscribe/1","title":"Solvent.subscribe/1","type":"function"},{"doc":"Subscribe to the event stream. The sink is what receives your events, and can be anything that implements the Solvent.Sink protocol, which includes anonymous functions, module-function-args tuples, and PIDs. Here's an example with a module-function-args tuple that subscribes to a single event type and identifies the subscription as &quot;My first subscriber&quot; . iex&gt; Solvent . subscribe ( ...&gt; { Solvent.MessengerHandler , :handle_event , [ ] } , ...&gt; types : [ &quot;com.example.event.published&quot; ] , ...&gt; id : &quot;My first subscriber&quot; ...&gt; ) { :ok , &quot;My first subscriber&quot; } This function shares its signature with Solvent.Subscription.new/2 , and creates a Solvent.Subscription in the same way, while also inserting the subscription into the Solvent.SubscriberStore so that the sink will begin to receive events. Sinks are given an event identifier , and not the event itself. Your sink must fetch the full event from the Solvent.EventStore , so that extra data is not copied between processes. You must also call Solvent.EventStore.ack/2 once you are done with the event, unless you want the event to stay in the event store forever. Tip Use the Solvent.Subscriber module to make a subscriber that automatically acknowledges events, along with lots of other nice features. You can also create a Solvent.Subscription struct yourself, and pass it to subscribe/1 .","ref":"Solvent.html#subscribe/2","title":"Solvent.subscribe/2","type":"function"},{"doc":"Remove a subscriber.","ref":"Solvent.html#unsubscribe/1","title":"Solvent.unsubscribe/1","type":"function"},{"doc":"The event that subscribers are waiting for. Events in Solvent follow the CloudEvents specification , and the keys in this struct match the attribute names given in the spec. See new/2 for valid values and their defaults.","ref":"Solvent.Event.html","title":"Solvent.Event","type":"module"},{"doc":"Create a new event. Override default values by providing them in opts . Options :id - the event's unique identifier. Default: a new version 4 UUID. :source - the entity that created this event. Default: &quot;Solvent&quot; . :type - also known as a topic, this is the name of the event that occurred. Set by the first argument to this function, but can be overridden. :data - information associated with the event. Default: nil :datacontenttype - a MIME type for the data in :data . Default: nil :dataschema - details on the structure of the value in :data . Default: nil :subject - describes the subject of the event, for example a filename for binary data in :data . Default: nil :time - a timestamp for when the event occurred. Default: DateTime.utc_now() at struct creation time. :extensions - a map of additional properties. Default: %{}","ref":"Solvent.Event.html#new/2","title":"Solvent.Event.new/2","type":"function"},{"doc":"Storage for events. This module delegates to the module given by Solvent's application configuration. By default, an ETS table is used for the event store. To change it, set the :event_store config for the :solvent app. config :solvent, event_store : Solvent.EventStore.ETS Any module configured in this way must implement the Solvent.EventStore.Base behaviour.","ref":"Solvent.EventStore.html","title":"Solvent.EventStore","type":"module"},{"doc":"The definition of an event store module. Modules using this behavior can be used with the Solvent.EventStore module to publish and fetch events.","ref":"Solvent.EventStore.Base.html","title":"Solvent.EventStore.Base","type":"behaviour"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:ack/2","title":"Solvent.EventStore.Base.ack/2","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:delete/1","title":"Solvent.EventStore.Base.delete/1","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:delete_all/0","title":"Solvent.EventStore.Base.delete_all/0","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:fetch/1","title":"Solvent.EventStore.Base.fetch/1","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:fetch!/1","title":"Solvent.EventStore.Base.fetch!/1","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:init/0","title":"Solvent.EventStore.Base.init/0","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#c:insert/2","title":"Solvent.EventStore.Base.insert/2","type":"callback"},{"doc":"","ref":"Solvent.EventStore.Base.html#t:event_id/0","title":"Solvent.EventStore.Base.event_id/0","type":"type"},{"doc":"","ref":"Solvent.EventStore.Base.html#t:subscriber_id/0","title":"Solvent.EventStore.Base.subscriber_id/0","type":"type"},{"doc":"ETS-based storage for Solvent.Event objects. Events are stored using insert/2 , along with an enumerable of subscriber IDs. The event is then stored until all subscribers call ack/2 to acknowledge the event, after which point the event is deleted from the store. The event store is initialized by the Solvent supervisor, so no setup work is necessary to use the store.","ref":"Solvent.EventStore.ETS.html","title":"Solvent.EventStore.ETS","type":"module"},{"doc":"Acknowledge that a listener has finished processing the event.","ref":"Solvent.EventStore.ETS.html#ack/2","title":"Solvent.EventStore.ETS.ack/2","type":"function"},{"doc":"Remove an event from storage.","ref":"Solvent.EventStore.ETS.html#delete/1","title":"Solvent.EventStore.ETS.delete/1","type":"function"},{"doc":"Delete all events from the event store.","ref":"Solvent.EventStore.ETS.html#delete_all/0","title":"Solvent.EventStore.ETS.delete_all/0","type":"function"},{"doc":"Fetch an event by {source, id} tuple. Returns {:ok, event} if the event exists, otherwise returns :error .","ref":"Solvent.EventStore.ETS.html#fetch/1","title":"Solvent.EventStore.ETS.fetch/1","type":"function"},{"doc":"Fetches an event by {source, id} tuple, and raises an error if it is not in the event store.","ref":"Solvent.EventStore.ETS.html#fetch!/1","title":"Solvent.EventStore.ETS.fetch!/1","type":"function"},{"doc":"Insert a new event into storage, along with all listeners that need to acknowledge it before it can be deleted. This does not activate any subscribers, use Solvent.publish/2 for that.","ref":"Solvent.EventStore.ETS.html#insert/2","title":"Solvent.EventStore.ETS.insert/2","type":"function"},{"doc":"A protocol for matching events. Filter expressions are built up from structs that implement this protocol, and they may be nested. Solvent uses a shorthand for building filter expressions from structs using keyword lists. In general, filter expressions are keyword lists, with the keys being a filter dialect, and the values being properties for that filter. Solvent supports all the filter dialects given in the WIP CloudEvents subscriptions spec, except for the sql dialect. Filters The exact filter compares its properties against an event's properties. [ exact : [ type : &quot;com.example.event&quot; , source : &quot;myapp&quot; ] ] The prefix filter specifies string values that an event's properties must start with. [ prefix : [ type : &quot;com.example.&quot; , subject : &quot;https://example.com/objects&quot; ] ] The suffix filter matches properties that end with the given strings. [ suffix : [ type : &quot;.created&quot; ] ] The all dialect lets you combine multiple filters, all of which must match. [ all : [ [ prefix : [ type : &quot;com.example.&quot; ] ] , [ exact : [ source : &quot;myapp&quot; ] ] ] ] The any filter works similarly, but will return true if any one of the given filters is true. [ any : [ [ exact : [ type : &quot;com.example.event&quot; ] ] , [ exact : [ type : &quot;com.example.other_event&quot; ] [ ] ] Finally, the not filter inverts the match of another filter. [ not : [ exact : [ type : &quot;com.example.event&quot; ] ] ]","ref":"Solvent.Filter.html","title":"Solvent.Filter","type":"protocol"},{"doc":"","ref":"Solvent.Filter.html#match?/2","title":"Solvent.Filter.match?/2","type":"function"},{"doc":"","ref":"Solvent.Filter.html#t:t/0","title":"Solvent.Filter.t/0","type":"type"},{"doc":"","ref":"Solvent.Filter.All.html","title":"Solvent.Filter.All","type":"module"},{"doc":"","ref":"Solvent.Filter.Any.html","title":"Solvent.Filter.Any","type":"module"},{"doc":"","ref":"Solvent.Filter.Exact.html","title":"Solvent.Filter.Exact","type":"module"},{"doc":"","ref":"Solvent.Filter.Not.html","title":"Solvent.Filter.Not","type":"module"},{"doc":"","ref":"Solvent.Filter.Prefix.html","title":"Solvent.Filter.Prefix","type":"module"},{"doc":"","ref":"Solvent.Filter.Suffix.html","title":"Solvent.Filter.Suffix","type":"module"},{"doc":"A protocol for delivering events to a target. By default, this protocol is implemented for: Tuple - Interpreted as a {module, function, args} tuple. The function is called with the event type and ID given as the first and second argument. PID - A message in the form of {:event, event_type, event_id} is sent to the PID. Function - The function is called with the arguments event_type and event_id","ref":"Solvent.Sink.html","title":"Solvent.Sink","type":"protocol"},{"doc":"","ref":"Solvent.Sink.html#deliver/3","title":"Solvent.Sink.deliver/3","type":"function"},{"doc":"","ref":"Solvent.Sink.html#t:t/0","title":"Solvent.Sink.t/0","type":"type"},{"doc":"A module shorthand for defining subscribers. Use this module to quickly create a module that can handle Solvent events. defmodule MyModule do use Solvent.Subscriber , filters : [ exact : [ type : &quot;myevent.published&quot; ] ] def handle_event ( type , event_id ) do # Fetch and handle your event here end end Then you only need to pass in the module name to Solvent.subscribe/1 , usually done in your application.ex , or wherever your code starts. By default, module subscribers will automatically call Solvent.EventStore.ack/2 once handle_event/2 returns. To disable this feature, set the :auto_ack option to false , and then you can acknowledge the event manually. This module provides an auto_ack/1 function that is compiled with the ID of your handler, so you only need to provide the event ID. defmodule MyModule do use Solvent.Subscriber , filters : [ prefix : [ type : &quot;myevents.&quot; ] ] , auto_ack : false def handle_event ( type , event_id ) do # Fetch and handle your event here ack_event ( event_id ) end end Using this module also imports Solvent.Subscriber.event!/1 which unwraps the result from Solvent.EventStore.fetch/1 and raises if the event is not found. Options :filters - the filter expression to match events against :id - the ID to give the subscriber function. Defaults to the current module name. :auto_ack - automatically call Subscriber.EventStore.ack/1 after handle_event/2 returns. Defaults to true .","ref":"Solvent.Subscriber.html","title":"Solvent.Subscriber","type":"behaviour"},{"doc":"Unwraps the result from Solvent.EventStore.fetch/1 and raises if the event is not found.","ref":"Solvent.Subscriber.html#event!/1","title":"Solvent.Subscriber.event!/1","type":"function"},{"doc":"Returns the filter to match events against. This function is automatically created when you use this module, but you can override it, if you need.","ref":"Solvent.Subscriber.html#c:filter/0","title":"Solvent.Subscriber.filter/0","type":"callback"},{"doc":"Performs an action when given an event type and event ID.","ref":"Solvent.Subscriber.html#c:handle_event/2","title":"Solvent.Subscriber.handle_event/2","type":"callback"},{"doc":"","ref":"Solvent.Subscriber.html#run_module/5","title":"Solvent.Subscriber.run_module/5","type":"function"},{"doc":"Returns the current subscriber ID. This function is automatically created when you use this module, but you can override it, if you need.","ref":"Solvent.Subscriber.html#c:subscriber_id/0","title":"Solvent.Subscriber.subscriber_id/0","type":"callback"},{"doc":"Describes a consumer's wish to receive events, as well as how to deliver them. See new/2 for instructions on how to create one of these structs, and then give that struct to Solvent.subscribe/1 to being receiving events.","ref":"Solvent.Subscription.html","title":"Solvent.Subscription","type":"module"},{"doc":"Test if a subscription should deliver a certain event.","ref":"Solvent.Subscription.html#match?/2","title":"Solvent.Subscription.match?/2","type":"function"},{"doc":"Create a new subscription struct. This function does not subscribe to the event stream, but once you create a Solvent.Subscription struct, you can give it to Solvent.subscribe/1 to subscribe to events. After providing a sink, you can additionally match for a source, multiple types, or provide filters to further match events. Examples To create a subscription for matching all events passing through Solvent, just provide a sink. iex&gt; Solvent.Subscription . new ( { MyModule , :handle_event , [ ] } ) To match all events coming from a single source, provide a :source option. It should be a string. iex&gt; Solvent.Subscription . new ( { MyModule , :handle_event , [ ] } , source : &quot;https://myapp.example.com&quot; ) You can also match a set of event types. iex&gt; Solvent.Subscription . new ( { MyModule , :handle_event , [ ] } , types : [ &quot;com.example.message.sent&quot; , &quot;com.example.message.received&quot; ] ) For more complex filtering, provide a struct implementing Solvent.Filter . You can build one from a keyword list with Solvent.build_filters/1 . iex&gt; Solvent.Subscription . new ( { MyModule , :handle_event , [ ] } , filter : Solvent . build_filters ( [ prefix : [ type : &quot;com.example.&quot; ] ] ) ) Multiple options can be provided in the same subscription. iex&gt; Solvent.Subscription . new ( { MyModule , :handle_event , [ ] } , source : &quot;https://myapp.example.com&quot; , types : [ &quot;com.example.message.sent&quot; , &quot;com.example.message.received&quot; ] )","ref":"Solvent.Subscription.html#new/2","title":"Solvent.Subscription.new/2","type":"function"}]